@echo off
setlocal

rem Default SWI-Prolog DOS executable, Windows executable and linker names
set DEFAULT_PROLOG_DOS=swipl
set DEFAULT_PROLOG_WINDOWS=swipl-win
set DEFAULT_PROLOG_LINKER=swipl-ld

rem File where SWI-Prolog filenames and paths will be saved
set FILE=.bplInfo

rem Searches for SWI-Prolog DOS executable
set PROLOG_DOS_LIST=plcon swipl plwin %DEFAULT_PROLOG_DOS%
for %%A in (%PROLOG_DOS_LIST%) do (
	set PROLOG=%%A
	%%A -g "halt." > nul 2> nul
	if not errorlevel 1 goto DOSFound
)

rem We couldn't find SWI-Prolog DOS executable
echo.
echo ERROR: SWI-Prolog DOS executable couldn't be find.
echo Please modify the 'bplInfo.bat' batch file and set the DEFAULT_PROLOG_DOS
echo variable with the command that starts the SWI-Prolog DOS-box version in
echo your system.
echo Example: set DEFAULT_PROLOG_DOS=swipl
echo.
exit /B 1


:DOSFound

rem Checks if SWI-Prolog is the 32 bit version
for /F "tokens=* delims= " %%A in ('%PROLOG% -dump-runtime-variables ^| findstr /B "PLARCH="') do (
	set PLARCH_AUX=%%A
)
set PLARCH=%PLARCH_AUX:~8,-2%
if %PLARCH% equ i386-win32 goto SWI32Found

rem We couldn't find SWI-Prolog 32 bit version
echo.
echo ERROR: SWI-Prolog 32 bit version couldn't be find in searcheable directories.
echo Please modify the PATH environment variable so that it can be found.
echo Maybe, the searcheable version we have found is the 64 bit instead.
echo Example: set PATH=%programfiles(x86)%\SWI-Prolog;%%PATH%%
echo.
exit /B 1

:SWI32Found

rem Searches for SWI-Prolog Windows executable
set PROLOG_WINDOWS_LIST=swipl-win plwin %DEFAULT_PROLOG_WINDOWS%
for %%A in (%PROLOG_WINDOWS_LIST%) do (
	set PROLOG_WINDOWS=%%A
	%%A -g "halt." > nul 2> nul
	if not errorlevel 1 goto WindowsFound
)

rem We couldn't find SWI-Prolog Windows executable
echo.
echo ERROR: SWI-Prolog Windows executable couldn't be find.
echo Please modify the 'bplInfo.bat' batch file and set the DEFAULT_PROLOG_WINDOWS
echo variable with the command that starts the SWI-Prolog Windows version in
echo your system.
echo Example: set DEFAULT_PROLOG_WINDOWS=swipl-win
echo.
exit /B 1


:WindowsFound

rem Searches for SWI-Prolog linker
set PROLOG_LINKER_LIST=swipl-ld plld %DEFAULT_PROLOG_LINKER%
for %%A in (%PROLOG_LINKER_LIST%) do (
	set PROLOG_LINKER=%%A
	%%A > nul 2> nul
	if not errorlevel 1 goto LinkerFound
)

rem We couldn't find SWI-Prolog linker
echo.
echo ERROR: SWI-Prolog linker couldn't be find.
echo Please modify the 'bplInfo.bat' batch file and set the DEFAULT_PROLOG_LINKER
echo variable with the command that starts the SWI-Prolog linker in your
echo system.
echo Example: set DEFAULT_PROLOG_LINKER=swipl-ld
echo.
exit /B 1


:LinkerFound

rem Launches SWI-Prolog to get its home directory and saves
rem all required information in ".bplInfo" file
%PROLOG% -g "current_prolog_flag(home, HomeDirectory), tell('%FILE%'), writef('%%w|%%w|%%w|%%w', [%PROLOG%, %PROLOG_WINDOWS%, %PROLOG_LINKER%, HomeDirectory]), told, halt."

rem Gets SWI-Prolog home directory from 4th argument in ".bplInfo" file
for /F "tokens=1-4* delims=|" %%A in (%FILE%) do (
	set PLDIR=%%D
)

REM We assume that the SWI-Prolog static library exists and comment the following:
REM rem Checks if SWI-Prolog static library exists
REM rem (library name is returned in a line with this format: PLLIB="name.lib";)
REM for /F "tokens=* delims= " %%A in ('%PROLOG% -dump-runtime-variables ^| findstr /B "PLLIB="') do (
REM 	set PLLIB_AUX=%%A
REM )
REM set PLLIB=%PLLIB_AUX:~7,-2%
REM if exist "%PLDIR%\lib\%PLLIB%" goto End

REM rem In some SWI-Prolog distributions, the main library needed by
REM rem SWI-Prolog linker hasn't the same name as the library that can be
REM rem found in the installation folder; here we copy the library so
REM rem SWI-Prolog linker won't fail
REM copy "%PLDIR%\lib\swipl.lib" "%PLDIR%\lib\%PLLIB%" > NUL 2> NUL
REM if not errorlevel 1 goto End

REM rem The library couldn't be copied; it's very likely that we're using
REM rem Windows Vista/7 and this task needs administrative privileges
REM echo In order for Bousi-Prolog to compile properly, the SWI-Prolog library must be
REM echo copied to the SWI-Prolog installation directory. This task will need adminis-
REM echo trative privileges.
REM echo.
REM echo Press any key to continue.
REM pause > nul
REM cd util
REM call elevate cmd /C copy "%PLDIR%\lib\swipl.lib" "%PLDIR%\lib\%PLLIB%"
REM cd ..
REM echo.
REM echo After the command prompt window is closed, press any key to continue.
REM echo.
REM pause > nul

:End

rem Writes script results
echo 	SWI-Prolog executables: %PROLOG% / %PROLOG_WINDOWS%
echo 	SWI-Prolog linker: %PROLOG_LINKER%
echo 	SWI-Prolog home directory: %PLDIR%

